{***************************************************************}
{*            RichView wrapper for QuickReport                 *}
{***************************************************************}
{*                                                             *}
{* Copyright (c) 2005-2011                                     *}
{*               Henk van Bokhorst / IVS Systemhouse B.V.      *}
{*             & QBS Software                                  *}
{* All rights reserved                                         *}
{*                                                             *}
{***************************************************************}
{*                                                             *}
{* Unit: RVQRUtils                                             *}
{*                                                             *}
{* Property editor for Text property                           *}
{*                                                             *}
{* 02/15/2011 First released version                           *}
{*                                                             *}
{***************************************************************}

unit RVQRContentLoader;

{$I RVQRCtrls.inc}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Buttons, DesignIntf, DesignEditors, RVScroll, RichView, CRVData,
  ExtCtrls, JPEG,
  {$IFDEF USEGIF}     GifImage, {$ENDIF}
  {$IFDEF USEGIF2007} GifImg,   {$ENDIF}
  {$IFDEF USEPNG}     PNGImage, {$ENDIF}
  {$IFDEF USEPNG2009} PNGImage, {$ENDIF}
  RVStyle;

type
  { This is the property editor for the Text property of TRVQRRichView }
  TRVQRContentEditor = class(TPropertyEditor)
  public
    procedure SetValue(const Value: string); override;
    function GetValue: string; override;
    function GetAttributes: TPropertyAttributes; override;
    procedure Edit; override;
  end;

  { This is the dialog form which is used to edit the Text property }
  TPropertyEditFrm = class(TForm)
    OpenDialog: TOpenDialog;
    Panel1: TPanel;
    Panel2: TPanel;
    Panel3: TPanel;
    Contents: TRichView;
    LoadButton: TButton;
    ClearButton: TButton;
    OKButton: TButton;
    CancelButton: TButton;
    ContentsStyle: TRVStyle;
    procedure FormCreate(Sender: TObject);
    procedure OKButtonClick(Sender: TObject);
    procedure LoadButtonClick(Sender: TObject);
    procedure ClearButtonClick(Sender: TObject);
  private
    TheDesigner: IDesigner;
    IsRVFFile, SomethingChanged: boolean;
    TextStream: TMemoryStream;
  public
  end;

implementation

uses RVQRCtrls, RVQRUtils;

{$R *.DFM}

{ TRVQRContentEditor }

procedure TRVQRContentEditor.Edit;

var
  PropertyEditFrm: TPropertyEditFrm;
  TextStream, TempStream: TMemorystream;
  TempList: TStringlist;

begin
  PropertyEditFrm := TPropertyEditFrm.Create(Application);
  try
    PropertyEditFrm.TheDesigner := Self.Designer;
    PropertyEditFrm.IsRVFFile   := Copy(TQRRichView(GetComponent(0)).Text, 1, 4) = '/rvf';
    if TQRRichView(GetComponent(0)).Text <> ''
    then
    begin
      TempList   := TStringlist.Create;
      TempStream := TMemorystream.Create;
      try
        if PropertyEditFrm.IsRVFFile
        then DecodeRVF(Copy(TQRRichView(GetComponent(0)).Text, 5, Length(TQRRichView(GetComponent(0)).Text) - 4), TempStream)
        else
        begin
          TempList.Text := TQRRichView(GetComponent(0)).Text;
          TempList.SaveToStream(TempStream);
        end;
        TempStream.Position := 0;
        PropertyEditFrm.Contents.Clear;
        if PropertyEditFrm.IsRVFFile
        then PropertyEditFrm.Contents.LoadRVFFromStream(TempStream)
        else PropertyEditFrm.Contents.LoadRTFFromStream(TempStream);
        PropertyEditFrm.Contents.Format;
      finally
        TempStream.Free;
        TempList.Free;
      end;
    end;
    PropertyEditFrm.ShowModal;
    if (PropertyEditFrm.ModalResult = mrOk) and PropertyEditFrm.SomethingChanged
    then
    begin
      if PropertyEditFrm.TextStream = nil
      then TQRRichView(GetComponent(0)).Text := ''
      else
      begin
        TempList := TStringlist.Create;
        TextStream := PropertyEditFrm.TextStream;
        TextStream.Position := 0;
        if PropertyEditFrm.IsRVFFile
        then
        begin
          TempStream := TMemorystream.Create;
          EncodeRVF(TextStream, TempStream);
          TempStream.Position := 0;
          TempList.LoadFromStream(TempStream);
          TempStream.Free;
          TempList.Text := '/rvf' + TempList.Text;
        end
        else TempList.LoadFromStream(TextStream);
        TextStream.Free;
        TQRRichView(GetComponent(0)).Text := TempList.Text;
        TempList.Free;
      end;
    end;
  finally
    PropertyEditFrm.Free;
  end;
end;

procedure TRVQRContentEditor.SetValue(const Value: string);
begin
  TQRRichView(GetComponent(0)).Text := Value;
end;

function TRVQRContentEditor.GetValue: string;
begin
  if TQRRichView(GetComponent(0)).Text = ''
  then Result := ''
  else Result := '(Text)';
end;

function TRVQRContentEditor.GetAttributes: TPropertyAttributes;
begin
  Result := [paDialog, paReadOnly];
end;

{ TPropertyEditFrm }

procedure TPropertyEditFrm.FormCreate(Sender: TObject);
begin
  TextStream       := nil;
  SomethingChanged := False;
end;

procedure TPropertyEditFrm.OKButtonClick(Sender: TObject);
begin
  if SomethingChanged then TheDesigner.Modified;
  ModalResult := mrOK;
end;

procedure TPropertyEditFrm.LoadButtonClick(Sender: TObject);
begin
  if OpenDialog.Execute
  then
  begin
    if Assigned(TextStream) then TextStream.Free;
    TextStream := TMemoryStream.Create;
    try
      Contents.Clear;
      Contents.DeleteUnusedStyles(True, True, True);
      Contents.BackgroundBitmap := nil;
      Contents.Format;
      SomethingChanged := True;
      TextStream.LoadFromFile(OpenDialog.FileName);
      IsRVFFile := IsRVF(TextStream);
      TextStream.Position := 0;
      if IsRVFFile
      then Contents.LoadRVFFromStream(TextStream)
      else Contents.LoadRTFFromStream(TextStream);
      Contents.Format;
    except
      TextStream.Free;
      TextStream := nil;
    end;
  end;
end;

procedure TPropertyEditFrm.ClearButtonClick(Sender: TObject);
begin
  if Assigned(TextStream)
  then
  begin
    TextStream.Free;
    TextStream := nil;
  end;
  Contents.Clear;
  Contents.DeleteUnusedStyles(True, True, True);
  Contents.BackgroundBitmap := nil;
  Contents.Format;
  SomethingChanged := True;
end;

initialization

  {$IFDEF USEGIF}     RegisterClasses([TGifImage]); {$ENDIF}
  {$IFDEF USEGIF2007} RegisterClasses([TGifImage]); {$ENDIF}
  {$IFDEF USEPNG}
    RegisterClass(TPngObject);
    RV_RegisterPngGraphic(TPngObject);
  {$ENDIF}
  {$IFDEF USEPNG2009}
    RegisterClass(TPngImage);
    RV_RegisterPngGraphic(TPngImage);
 {$ENDIF}

end.
